#ifndef GAMECOREH
#define GAMECOREH

#include "display.h"
#include "terrain.h"
#include "console.h"
#include "menu.h"
#include "clock.h"
#include "skeleton.h"
#include "physics.h"
#include "tankobject.h"
#include "object.h"
#include "missiles.h"
#include "mirvs.h"

#include <map>
#include <vector>

class GameCore
{
	Physics physics;
	Display disp;
	Console console;
	vertex position;
	vertex cameraVelocity;
	vertex rotation;
	vertex light;
	Terrain terrain;
	
	SkeletonNode skeleton;
	SkeletonNode sphere;
	SkeletonNode explosion;
	SkeletonNode skelBackground;
	
	//TankObject *tank;
	std::list<TankObject *>::iterator tank;
	
	std::map<std::string, SkeletonNode *> jointTable;

	/* These maps are ruthlessly abused.  They could be used for the initial lookup, but after that, they're bad. */
	std::map<std::string, int> variableTable;
	std::map<std::string, Mesh> meshMap;
	unsigned int imageID;
	unsigned int font;
	unsigned int texExplosion;
	unsigned int texMissile;
	unsigned int texTerrain[3][5];
	unsigned int texSelect;
	unsigned int texBackground;
	unsigned int texSplashScreen;
	bool keyTable[256];
	bool consoleActive;
	bool splashScreen;
	bool displayMessage;
	unsigned int activeMenu;
	void processCommand(const std::string &command);
	void processCommandList(const std::string &commandList);
	unsigned int loadImage(const char *file);
	void placeTanks(int number);
	void getJoints();

	unsigned int frames;
	Clock timer;
	Clock idleClock;
	float fps;
	float aimPhi;
	float aimTheta;
	std::string sAimPhi;
	std::string sAimTheta;
	std::string sVelocity;
	std::string sHealth;
	std::list<Object *> objects;
	std::string message;
	float velocity;
	std::list<Weapon *> weapons;
	std::list<Weapon *>::iterator selectedWeapon;

	ExplosiveWeapon missile;

	std::vector<MenuData> menu;
	std::list<TankObject *> tanks;
public:
	GameCore();
	~GameCore();
	void keypress(unsigned char key, bool down, int modifiers);
	void mouseMotion(bool down, float x, float y, float xrel, float yrel);
	void mouseButton(int button, bool down, float x, float y);
	void idle();
	void resize(int width, int height);
	void display();
};

#endif
