#include "mesh.h"

#include <fstream>
#include <string>
#include <iostream>

bool load(const char *filename, Mesh &mesh)
{
	mesh.vertices.resize(0);
	mesh.color.resize(0);
	mesh.normal.resize(0);
	mesh.texture.resize(0);
	mesh.triangle.resize(0);

	std::ifstream file(filename);
	std::string token;
	int size;

	if (!file)
	{
		std::cerr << "Unable to open file: " << filename << std::endl;
		return false;
	}

	std::cout << "File opened." << std::endl;

	if (!(file >> size) || !size)
	{
		std::cerr << "Error loading model " << filename << ".  Invalid size: " << size << std::endl;
		return false;
	}

	while (file >> token)
	{
		if (token == "vertices")
		{
			std::cout << "Loading vertices." << std::endl;
			mesh.vertices.resize(size);
			int n = -1;
			float f[3];
			while (++n < size && file >> f[0] >> f[1] >> f[2])
			{
				mesh.vertices[n].x = f[0];
				mesh.vertices[n].y = f[1];
				mesh.vertices[n].z = f[2];
			}
			std::cout << "Vertices loaded." << std::endl;
		}
		else if (token == "texture")
		{
			std::cout << "Loading texture coordinates." << std::endl;
			mesh.texture.resize(size);
			int n = -1;
			float f[2];
			while (++n < size && (file >> f[0] >> f[1]))
			{
				mesh.texture[n].x = f[0];
				mesh.texture[n].y = f[1];
			}
			std::cout << "Loading completed." << std::endl;
		}
		else if (token == "color")
		{
			std::cout << "Loading color." << std::endl;
			mesh.color.resize(size);
			int n = -1;
			float f[3];
			while (++n < size && file >> f[0] >> f[1] >> f[2])
			{
				mesh.color[n].x = f[0];
				mesh.color[n].y = f[1];
				mesh.color[n].z = f[2];
			}
			std::cout << "Color loaded." << std::endl;
		}
		else if (token == "normals")
		{
			std::cout << "Loading normals." << std::endl;
			mesh.normal.resize(size);
			int n = -1;
			float f[3];
			while (++n < size && file >> f[0] >> f[1] >> f[2])
			{
				mesh.normal[n].x = f[0];
				mesh.normal[n].y = f[1];
				mesh.normal[n].z = f[2];
			}
			std::cout << "Normals loaded." << std::endl;
		}
		else if (token == "quadrilaterals")
		{
			std::cout << "Loading quads." << std::endl;
			int count;
			int pos = int(mesh.triangle.size());
			if (!(file >> count))
			{
				std::cerr << "Error loading mesh.  Invalid count." << std::endl;
				return false;
			}
			mesh.triangle.resize(pos + count * 3 * 2);
			int f[4];
			while (count-- && file >> f[0] >> f[1] >> f[2] >> f[3])
			{
				if (f[0] > size || f[1] > size || f[2] > size || f[3] > size)
				{
					std::cerr << "Error loading mesh.  Invalid index." << std::endl;
					return false; // We don't want to index invalid data.
				}
				mesh.triangle[pos++] = f[0];
				mesh.triangle[pos++] = f[1];
				mesh.triangle[pos++] = f[2];

				mesh.triangle[pos++] = f[2];
				mesh.triangle[pos++] = f[3];
				mesh.triangle[pos++] = f[0];
			}
			std::cout << "Quads loaded." << std::endl;
		}
		else if (token == "triangles")
		{
			std::cout << "Loading triangles." << std::endl;
			int count;
			int pos = int(mesh.triangle.size());
			if (!(file >> count))
			{
				std::cerr << "Error loading mesh. Invalid count." << std::endl;
				return false;
			}
			mesh.triangle.resize(pos + count * 3);
			int f[3];
			while (count-- && file >> f[0] >> f[1] >> f[2])
			{
				if (f[0] > size || f[1] > size || f[2] > size)
				{
					std::cerr << "Error loading mesh. Invalid index." << std::endl;
					return false; // We don't want to index invalid data.
				}
				mesh.triangle[pos++] = f[0];
				mesh.triangle[pos++] = f[1];
				mesh.triangle[pos++] = f[2];
			}
			std::cout << "Triangle loaded." << std::endl;
		}
		else
		{
			std::cout << "Unrecognized token." << std::endl;
			return false;
		}
	}

	std::cout << "Mesh loaded." << std::endl;
	return true;
}
