#include "mirvs.h"

#include "image.h"

void MIRVObject::explode(Physics::Environment env)
{
	vertex x = {1, 0, 0};
	vertex z = {0, 0, 1};
	deleted = true;
	child.fire(*env.objects, position, velocity);
	child.fire(*env.objects, position, velocity + x * 4.0f);
	child.fire(*env.objects, position, velocity + x * -4.0f);
	child.fire(*env.objects, position, velocity + z * 4.0f);
	child.fire(*env.objects, position, velocity + z * -4.0f);
}

void MIRVObject::contactEvent(Physics::Environment env)
{
	std::cout << "MIRVObject::contactEvent" << std::endl;
	explode(env);
}

void MIRVObject::advanceEvent(Physics::Environment env, float advanceTime)
{
	if (velocity.y < 0)
		explode(env);
}

bool MIRV::load(std::map<std::string, Mesh> &meshMap, Display &display)
{
	Image image;
	bool result = true;
	result &= ::load("data/giraffe.png", image);
	explosiveTexture = display.load(image);
	result &= ::load("data/weapons/mirv.png", image);
	iconTexture = display.load(image);
	result &= missile.load(meshMap, display);
	result &= ::load("data/sphere.skel", explosiveSkeleton);

	return result;
}

void MIRV::fire(std::list<Object *> &objects, const vertex &position, const vertex &velocity)
{
	MIRVObject *o = new MIRVObject;
	o->skeleton = explosiveSkeleton;
	o->texture = explosiveTexture;
	o->position = position;
	o->velocity = velocity;
	o->child = missile;
	
	objects.push_back(o);
}

void DeathsHeadObject::explode(Physics::Environment env)
{
	vertex x = {1, 0, 0};
	vertex z = {0, 0, 1};
	deleted = true;
	child.fire(*env.objects, position, velocity);
	child.fire(*env.objects, position, velocity + z * -4.0f);
	child.fire(*env.objects, position, velocity + z * 4.0f);
	child.fire(*env.objects, position, velocity + x * -4.0f);
	child.fire(*env.objects, position, velocity + x * 4.0f);
	child.fire(*env.objects, position, velocity + x * -4.0f + z * -4.0f);
	child.fire(*env.objects, position, velocity + x * -4.0f + z * 4.0f);
	child.fire(*env.objects, position, velocity + x * 4.0f + z * -4.0f);
	child.fire(*env.objects, position, velocity + x * 4.0f + z * 4.0f);

}

void DeathsHeadObject::contactEvent(Physics::Environment env)
{
	std::cout << "DeathsHeadObject::contactEvent" << std::endl;
	explode(env);
}

void DeathsHeadObject::advanceEvent(Physics::Environment env, float advanceTime)
{
	if (velocity.y < 0)
		explode(env);
}

bool DeathsHead::load(std::map<std::string, Mesh> &meshMap, Display &display)
{
	Image image;
	bool result = true;
	result &= ::load("data/giraffe.png", image);
	explosiveTexture = display.load(image);
	result &= ::load("data/weapons/deathshead.png", image);
	iconTexture = display.load(image);
	result &= payload.load(meshMap, display);
	result &= ::load("data/sphere.skel", explosiveSkeleton);

	return result;
}

void DeathsHead::fire(std::list<Object *> &objects, const vertex &position, const vertex &velocity)
{
	DeathsHeadObject *o = new DeathsHeadObject;
	o->skeleton = explosiveSkeleton;
	o->texture = explosiveTexture;
	o->position = position;
	o->velocity = velocity;
	o->child = payload;
	
	objects.push_back(o);
}
