#ifndef OBJECTH
#define OBJECTH

#include "vertex.h"
#include "skeleton.h"
#include "physics.h"

class Object
{
	class ID
	{
		static unsigned int nextID;
		unsigned int objectID;
	public:
		ID() : objectID(++nextID) {}
		ID(const ID &newID) : objectID(++nextID) {/* don't assign anything */}
		unsigned int id() { return objectID; }
		const ID &operator=(const ID &a) { objectID = nextID++; return *this; }
	};
	ID objectID;
public:
	Object();
	virtual ~Object() {}

	unsigned int id() { return objectID.id(); }
	
	vertex position;
	vertex scale;
	vertex rotation;

	float radius;
	float mass;
	float elasticity;
	float timeAlive;

	vertex velocity;
	
	// Here are a few state variables...
	bool resting; // Sitting on the terrain
	bool active; // Does stuff
	bool deleted; // Will be deleted the next chance it gets
	bool fixed; // Stays exactly where it is
	bool immobile; // Can't move, but might fall
	
	// I'm not sure if this belongs here, but it seems to simplify things at the moment.
	SkeletonNode skeleton;
	unsigned int texture;

	// Events
	virtual void contactEvent(Physics::Environment env);
	virtual void advanceEvent(Physics::Environment env, float advanceTime);
	virtual void damageEvent(Physics::Environment env, float damage);

	// player/owner will probably be important
};

#endif
