#ifndef TERRAINH
#define TERRAINH

#include "matrix.h"
#include "vertex.h"

#include <list>

struct TerrainValue
{
	float height;
	vertex normal;
	
	unsigned int texture;
	int orientation;
};

class Terrain : public Matrix<TerrainValue>
{
public:
struct TextureInfo
	{
		float height;
		unsigned int textures[5]; // = {solid, one quarter, one half, three quarters, 1/2 diagonal}
	};
private:
	vertex terrainScale;
	std::list<TextureInfo> textureInfo;
	vertex surfaceNormal(unsigned int x, unsigned int y, bool one);
public:
	Terrain(unsigned int width = 64, unsigned int height = 64, float xScale = 1.0f, float yScale = 1.0f, float zScale = 1.0f);
	void clear(float zero);
	void generate(int iterations, float floor, float ceiling);
	void generateNormals(unsigned int left, unsigned int bottom, unsigned int right, unsigned int top);
	void generateTexture(unsigned int left, unsigned int bottom, unsigned int right, unsigned int top);
	void setTextureInfo(const std::list<TextureInfo> &tInfo) { textureInfo = tInfo; }
	void scale(float x, float y, float z);
	vertex scale();
	
	// Additions to improve collision detection with the ground.
	float maxHeight(unsigned int x, unsigned int y);
	float minHeight(unsigned int x, unsigned int y);

	void sphereDeform(const vertex &position, float radius);
};

#endif
