#include "window.h"
#include "exception.h"

#ifdef __APPLE__
#include <Glut/glut.h>
#else
#include <GL/glut.h>
#endif

Window *Window::currentWindow = 0;

Window::Window(int &argc, char *argv[], const char *caption, int newWidth, int newHeight)
: width(newWidth), height(newHeight)
{
	std::cout << "Window::Window()" << std::endl;

	leftDown = middleDown = rightDown = down = false;

	glutInit(&argc, argv);
	std::cout << "After init" << std::endl;
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH);
	std::cout << "init display mode" << std::endl;
	glutInitWindowSize(width, height);
	std::cout << "size" << std::endl;

	currentWindow = this;
	std::cout << "Doing stuff." << std::endl;
	windowID = glutCreateWindow(caption);
	std::cout << "Window created." << std::endl;
	glutDisplayFunc(staticDisplay);
	glutIdleFunc(staticIdle);
	glutReshapeFunc(staticReshape);
	glutKeyboardFunc(staticKeyboard);
	glutKeyboardUpFunc(staticKeyboardUp);
	glutSpecialFunc(staticSpecial);
	glutSpecialUpFunc(staticSpecialUp);
	glutMouseFunc(staticMouse);
	glutMotionFunc(staticMotion);
}

Window::~Window()
{
	cout << "Window::~Window()" << endl;
}

void Window::eventLoop(GameCore *newGame)
{
	game = newGame;
	glutMainLoop();
}

void Window::display()
{
	if (game)
		game->display();
	glutSwapBuffers();
}

void Window::reshape(int width, int height)
{
	if (game)
		game->resize(width, height);
	glutPostRedisplay();
}

void Window::keyboard(unsigned char key, int x, int y, bool down)
{
	if (game)
		game->keypress(key, down, glutGetModifiers());
}

void Window::idle()
{
	if (game)
		game->idle();
	glutPostRedisplay();
}

void Window::special(int key, int x, int y, bool down)
{
	if (game)
	{
		unsigned char ch = 0;
		switch (key)
		{
		case GLUT_KEY_LEFT: ch = TWK_LEFT; break;
		case GLUT_KEY_UP: ch = TWK_UP; break;
		case GLUT_KEY_RIGHT: ch = TWK_RIGHT; break;
		case GLUT_KEY_DOWN: ch = TWK_DOWN; break;
		case GLUT_KEY_PAGE_UP: ch = TWK_PAGEUP; break;
		case GLUT_KEY_PAGE_DOWN: ch = TWK_PAGEDOWN; break;
		}
		if (ch)
			game->keypress(ch, down, glutGetModifiers());
	}
}

void Window::mouse(int button, int state, int x, int y)
{
	lastX = x;
	lastY = y;
	down = (state == GLUT_DOWN);
}

void Window::motion(int x, int y)
{
	if (game)
		game->mouseMotion(down, static_cast<float>(x) / width, static_cast<float>(y) / height, static_cast<float>(x - lastX) / width, static_cast<float>(y - lastY) / height);

	lastX = x;
	lastY = y;
}
